from setuptools import setup, find_packages

import pathlib
# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name='obsidian-progress-report',
    description='A simple command line app for generating progress report for contributions in the Obsidian vault',
    version='0.1.1',
    packages=find_packages(),  # list of all packages
    python_requires='>=3.8',
    entry_points='''
        [console_scripts]
        obsidian-progress-report=obsidian_progress_report.__main__:main
    ''',
    author="Yuliya Bagriy",
    long_description=README,
    long_description_content_type="text/markdown",
    license='Unlicense',
    url='https://github.com/aviskase/obsidian-progress-report',
    download_url='https://github.com/aviskase/obsidian-progress-report/archive/0.1.1.tar.gz',
    author_email='aviskase@gmail.com',
    classifiers=[
        "License :: Public Domain",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ]
)



import argparse
import calendar
from datetime import date
from pathlib import Path

from obsidian_progress_report.report import generate_report


def dir_path(path: str) -> Path:
    if Path(path).is_dir():
        return Path(path).absolute()
    else:
        raise argparse.ArgumentTypeError(f'{path} is not a valid directory path')


def date_arg(d: str) -> date:
    try:
        return date.fromisoformat(d)
    except ValueError:
        raise argparse.ArgumentTypeError(f'Given date \'{d}\' is not valid')


def main():
    parser = argparse.ArgumentParser(
        description='A simple command line app for generating progress report for contributions in the Obsidian vault.',
    )
    parser.add_argument(dest='vault', type=dir_path, help='Path where to find the Obsidian vault.')
    parser.add_argument(
        '-o', '--output', default='.',
        help='Where to save the generated report (relative to the vault\'s root). If not specified, root will be used.'
    )
    current_day: date = date.today()
    start_of_month = current_day.replace(day=1)
    _, last_day = calendar.monthrange(current_day.year, current_day.month)
    end_of_month = current_day.replace(day=last_day)

    parser.add_argument(
        '-s', '--start', type=date, default=start_of_month,
        help='Start of report period, by default is set to the start of current month. Format: "YYYY-MM-DD".'
    )
    parser.add_argument(
        '-e', '--end', type=date, default=end_of_month,
        help='End of report period, by default is set to the end of current month. Format: "YYYY-MM-DD".'
    )
    parser.add_argument(
        '-i', '--include', choices=['created', 'updated', 'all'], default='all',
        help='Which sections report should contain.'
    )
    args = parser.parse_args()
    output_dir: Path = (args.vault / args.output).absolute()
    print(f'Creating output directory: {output_dir}')
    output_dir.mkdir(parents=True, exist_ok=True)
    print('Start report generation...')
    generate_report(args.vault, output_dir, args.start, args.end, args.include)
    print('Report is generated!')


if __name__ == '__main__':
    main()



from typing import Tuple, List
from pathlib import Path
from datetime import date


def generate_report(vault: Path, output_dir: Path, start: date, end: date, sections):
    created, updated = gather_files(vault, start, end)
    report = Report(vault, output_dir, start, end)
    print(f'Report will be saved to file: {report.path}')
    report.add_title(f'{start.isoformat()} – {end.isoformat()}')
    if sections in ['all', 'created']:
        print('Adding created files...')
        report.add_section('Created', created)
    if sections in ['all', 'updated']:
        print('Adding updated files...')
        report.add_section('Updated', updated)


def gather_files(dir: Path, start: date, end: date) -> Tuple[List[Path], List[Path]]:
    created = []
    updated = []
    files = sorted(dir.rglob('*.md'))
    for f in files:
        if '.trash' in f.parts:
            continue
        create_time = date.fromtimestamp(f.stat().st_ctime)
        update_time = date.fromtimestamp(f.stat().st_mtime)
        if start <= create_time <= end:
            created.append(f)
        elif start <= update_time <= end:
            updated.append(f)
    return created, updated


class Report:
    def __init__(self, vault: Path, output_dir: Path, start: date, end: date):
        report_path = output_dir / f'{start.isoformat()} - {end.isoformat()}.md'
        report_path.touch(exist_ok=True)
        self.path = report_path
        self.vault = vault

    def add_title(self, title: str):
        with self.path.open(mode='w') as report_file:
            report_file.write(f'# {title}\n\n')

    def add_section(self, heading: str, files: List[Path]):
        with self.path.open(mode='a') as report_file:
            report_file.write(f'## {heading}\n\n')
            if files:
                report_file.writelines(map(
                    self.path_to_list_item,
                    files
                ))
                report_file.write('\n\n')
            else:
                report_file.write('No files.\n\n')

    def path_to_list_item(self, path: Path) -> str:
        path_link = str(path.relative_to(self.vault).with_suffix(''))
        return f'- [[{path_link}]]\n'
